<?php

namespace Drupal\bootstrap_horizontal_tabs\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html as HtmlUtility;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Template\Attribute;

/**
 * Plugin implementation of the 'bootstrap_horizontal_tabs' formatter.
 *
 * @FieldFormatter(
 *   id = "bootstrap_horizontal_tabs",
 *   label = @Translation("Horizontal Tabs"),
 *   field_types = {
 *     "bootstrap_horizontal_tabs"
 *   }
 * )
 */
class BootstrapHorizontalTabs extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'tab_display' => 'tabs',
      'tab_orientation' => 'horizontal',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['tab_display'] = [
      '#type' => 'select',
      '#title' => $this->t('Header display'),
      '#options' => [
        'tabs' => $this->t('Tabs'),
        'pills' => $this->t('Pill buttons'),
      ],
      '#default_value' => $this->getSetting('tab_display'),
    ];
    $elements['tab_orientation'] = [
      '#type' => 'select',
      '#title' => $this->t('Header orientation'),
      '#options' => [
        'horizontal' => $this->t('Horizontal'),
        'vertical' => $this->t('Vertical'),
      ],
      '#default_value' => $this->getSetting('tab_orientation'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $settings = $this->getSettings();

    if (!empty($settings['tab_display'])) {
      $summary[] = t('Header display: @display', ['@display' => $settings['tab_display']]);
    }
    if (!empty($settings['tab_orientation'])) {
      $summary[] = t('Header orientation: @display', ['@display' => $settings['tab_orientation']]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [
      '#theme' => 'field__bootstrap_horizontal_tabs',
    ];

    // Convert settings to values for use.
    $settings = $this->getSettings();
    $tab_display = $settings['tab_display'] ?? 'tabs';
    $tab_orientation = $settings['tab_orientation'] === 'vertical' ? 'flex-column' : '';

    // Generate a unique id for the tabs instance.
    $instance_id = HtmlUtility::getUniqueId('bootstrap-horizontal-tabs');

    // Create new tabs wrapper attributes object for use in the template.
    $tabs_wrapper_attributes = [
      'class' => [
        'nav',
        'nav-' . $tab_display,
        $tab_orientation,
      ],
      'id' => $instance_id,
      'role' => 'tablist',
    ];
    $elements['#tabs_wrapper_attributes'] = new Attribute($tabs_wrapper_attributes);

    // Create new content wrapper attributes object for use in the template.
    $content_wrapper_attributes = [
      'class' => [
        'tab-content',
      ],
      'id' => $instance_id . '-content',
    ];
    $elements['#content_wrapper_attributes'] = new Attribute($content_wrapper_attributes);

    // Process each field item.
    foreach ($items as $delta => $item) {
      $item_id = preg_replace('/[^a-z0-9_]+/', '_', strtolower($item->header));
      $processed_body = [
        '#type' => 'processed_text',
        '#text' => $item->body_value,
        '#format' => $item->body_format,
        '#langcode' => $item->getLangcode(),
      ];
      $elements[$delta] = [
        'header' => $item->header,
        'body' => $processed_body,
      ];

      // Create new list item attributes object for use in the template.
      $list_item_attributes = [
        'class' => [
          'nav-item',
        ],
        'role' => 'tab',
      ];
      $elements[$delta]['list_item_attributes'] = new Attribute($list_item_attributes);

      // Create new link attributes object for use in the template.
      $link_attributes = [
        'aria-selected' => 'false',
        'class' => [
          'nav-link',
        ],
        'data-toggle' => 'tab',
        'href' => '#' . $item_id,
        'id' => $item_id . '-tab',
        'role' => 'tab',
      ];
      $elements[$delta]['link_attributes'] = new Attribute($link_attributes);

      // Create new content attributes object for use in the template.
      $content_attributes = [
        'aria-labelledby' => $item_id . '-tab',
        'class' => [
          'tab-pane',
          'fade',
        ],
        'id' => $item_id,
        'role' => 'tabpanel',
      ];
      $elements[$delta]['content_attributes'] = new Attribute($content_attributes);

      // Add defaut classes and attributes to first item upon initial render.
      if ($delta === 0) {
        $elements[$delta]['list_item_attributes']->addClass('active');
        $elements[$delta]['link_attributes']->addClass('active');
        $elements[$delta]['link_attributes']->setAttribute('aria-selected', 'true');
        $elements[$delta]['content_attributes']->addClass('active');
        $elements[$delta]['content_attributes']->addClass('show');
      }
    }

    return $elements;
  }

}
